Attribute VB_Name = "Constraint"
'  FILE:       Constraint.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality to add, modify, delete
'              and edit constraints. Also other functions specific
'              to connection strings.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Constants for the different constraint operations
' Note : They correspond to the index of the image in the
' toolbar
Public Const gintAdd = 1
Public Const gintDelete = 2
Public Const gintMoveUp = 3
Public Const gintMoveDown = 4

' The different constraint types are defined as a public enum
' in cConstraint - they correspond to the index of all the
' controls on the corresponding tabs on the frmSteps form

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "Constraint."

Private cPreStepIds As cVectorLng
Private cPostStepIds As cVectorLng

Public Sub ConstraintCleanup()

    If Not cPreStepIds Is Nothing Then
        cPreStepIds.Clear
    End If
    Set cPreStepIds = Nothing
    
    If Not cPostStepIds Is Nothing Then
        cPostStepIds.Clear
    End If
    Set cPostStepIds = Nothing

End Sub

Public Sub ConstraintInit()

    Set cPreStepIds = New cVectorLng
    Set cPostStepIds = New cVectorLng
    
End Sub

Public Sub ConstraintCommand(intMethod As Integer)

    On Error GoTo ConstraintCommandErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    ' Calls a procedure to execute the command selected
    If frmSteps.sstStepProperties.Tab = gintPreExecutionSteps Then
        Call ExecuteConstraintMethod( _
                frmSteps.lstGlobals(gintPreStep), _
                frmSteps.lstConstraints(gintPreStep), _
                intMethod)
    Else
        Call ExecuteConstraintMethod( _
                frmSteps.lstGlobals(gintPostStep), _
                frmSteps.lstConstraints(gintPostStep), _
                intMethod)
    End If

    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
ConstraintCommandErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    BugMessage gstrSource & _
            " : Unable to execute constraint method : " & _
            Str(intMethod)
    ' Reset the mousepointer
    Call ShowFree
    gstrSource = mstrModuleName & "ConstraintCommand"
    ' Display a friendly error message!
    ShowError errConstraintCommandFailed
    
End Sub
Private Sub InsertNewConstraints(lstConstraints As ListBox, _
        lstGlobals As ListBox, _
        ByVal cStepRecord As cStep, _
        ByVal intConstraintType As Integer)
    
    Dim intIndex As Integer
    Dim cConsRec As cConstraint
    Dim cGlobalConstraintStep As cStep
    Dim cArrStepIds As cVectorLng
    
    On Error GoTo InsertNewConstraintsErr
    gstrSource = mstrModuleName & "InsertNewConstraints"
    
    If intConstraintType = gintPreStep Then
        Set cArrStepIds = cPreStepIds
    Else
        Set cArrStepIds = cPostStepIds
    End If
    
    ' Now insert all new constraint records
    For intIndex = 0 To lstConstraints.ListCount - 1
    
        ' All constraints that were loaded by the application
        ' before displaying the step properties will have the
        ' itemdata set to the constraint id
        If lstConstraints.ItemData(intIndex) = 0 Then
            Set cConsRec = New cConstraint
            cConsRec.StepId = cStepRecord.StepId
            cConsRec.VersionNo = cStepRecord.VersionNo
            cConsRec.WorkspaceId = cStepRecord.WorkspaceId
            
            ' Query the global step details
            Set cGlobalConstraintStep = New cStep
            
            ' Retrieve the step id from the array that was
            ' maintained in parallel with the constraint list
            cGlobalConstraintStep.StepId = cArrStepIds(intIndex)
            
            Set cGlobalConstraintStep = gcSteps.QueryStep _
                    (cGlobalConstraintStep.StepId)
            cConsRec.GlobalStepId = cGlobalConstraintStep.StepId
            cConsRec.GlobalVersionNo = cGlobalConstraintStep.VersionNo
            Set cGlobalConstraintStep = Nothing
            
            cConsRec.ConstraintType = intConstraintType
            cConsRec.SequenceNo = intIndex
            
            ' Add the constraint to the the array
            gcConstraints.Add cConsRec
            Set cConsRec = Nothing
            
        End If
    Next intIndex
        
    Exit Sub
    
InsertNewConstraintsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "InsertNewConstraints"
    Err.Raise vbObjectError + errInsertNewConstraintsFailed, _
            gstrSource, _
            LoadResString(errInsertNewConstraintsFailed)
    
End Sub
Public Sub AddGlobalConstraint(ByVal cStepRecord As cStep, _
        ByVal intConstraintType As Integer)
    ' Inserts a constraint record for the passed in global step
    
    Dim cConsRec As cConstraint
    
    On Error GoTo AddGlobalConstraintErr
    
    Set cConsRec = New cConstraint
    
    ' Initialize the global step id and version number from
    ' the passed in global step record
    cConsRec.GlobalStepId = cStepRecord.StepId
    cConsRec.GlobalVersionNo = cStepRecord.VersionNo
    cConsRec.WorkspaceId = cStepRecord.WorkspaceId
    
    ' The step id and version number will be zero,
    ' since this constraint is not for a particular
    ' step, but for the entire workspace
    cConsRec.StepId = 0
    cConsRec.VersionNo = gstrMinVersion
    
    cConsRec.ConstraintType = intConstraintType
    
    ' Initialize the sequence for the constraint to zero
    cConsRec.SequenceNo = 0
    
    ' Add the constraint to the the array
    gcConstraints.Add cConsRec
    
    Set cConsRec = Nothing
            
    Exit Sub
    
AddGlobalConstraintErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "AddGlobalConstraint"
    Err.Raise vbObjectError + errInsertNewConstraintsFailed, _
            gstrSource, _
            LoadResString(errInsertNewConstraintsFailed)
    
End Sub
Public Sub CopyConstraintRecords(ByVal cStepFrom As cStep, _
        ByVal cStepTo As cStep)
    ' Creates constraint records for the StepTo class
    ' that are identical to existing constraint record
    ' for StepFrom
    
    Dim cConsToCreate As cConstraint
    Dim intIndex As Integer
    Dim vntConstraints As Variant
    
    On Error GoTo CopyConstraintRecordsErr
    
    ' Constraints can be copied from one step to another
    ' only if both the steps belong to the same workspace
    ' This ensures that the same global steps qualify as
    ' pre or post-execution constraints for the new step
    ' being created
    If Not IsGlobal(StepClass:=cStepFrom) Then
        If cStepFrom.WorkspaceId <> cStepTo.WorkspaceId Then
            Call ShowMessage(errNoConstraintsCreated)
            Exit Sub
        End If
    End If
    
    ' Call a function that will return all the constraints
    ' that have been defined for the step
    If Not IsGlobal(StepClass:=cStepFrom) Then
        vntConstraints = gcConstraints.ConstraintsForStep( _
                cStepFrom.StepId, cStepFrom.VersionNo, _
                blnSort:=False)
    Else
        vntConstraints = gcConstraints.ConstraintsForStep( _
                cStepFrom.StepId, cStepFrom.VersionNo, _
                blnSort:=False, blnGlobal:=True, _
                blnGlobalConstraintsOnly:=True)
    End If
    
    If Not IsEmpty(vntConstraints) And IsArray(vntConstraints) Then
        For intIndex = LBound(vntConstraints) To UBound(vntConstraints)
                
            ' Initialize the new constraint record to the
            ' existing step constraint
            Set cConsToCreate = vntConstraints(intIndex).Clone
            
            ' Copy the step id and version number from the
            ' new step record being created
            If IsGlobal(StepClass:=cStepFrom) Then
                cConsToCreate.GlobalStepId = cStepTo.StepId
                cConsToCreate.GlobalVersionNo = cStepTo.VersionNo
            Else
                cConsToCreate.StepId = cStepTo.StepId
                cConsToCreate.VersionNo = cStepTo.VersionNo
            End If
            cConsToCreate.WorkspaceId = cStepTo.WorkspaceId
            
            ' Add the new constraint record
            gcConstraints.Add cConsToCreate
            Set cConsToCreate = Nothing
            
        Next intIndex
    End If
    
    Exit Sub
    
CopyConstraintRecordsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCopyFailed, mstrModuleName & "CopyConstraintRecords", _
        LoadResString(errCopyFailed)
    
End Sub
Public Sub WriteConstraints(lstConstraints As ListBox, _
        lstGlobals As ListBox, _
        ByVal cStepRecord As cStep)

    ' Commits the constraint records to the database
    
    Dim cConsRec As cConstraint
    Dim intConstraintType As Integer
    Dim intIndex As Integer
    Dim intConsIndex As Integer
    Dim vntConstraints As Variant
    
    On Error GoTo WriteConstraintsErr
    
    intConstraintType = lstConstraints.Index
    
    ' Call a function that will return all the Pre- or post-
    ' execution constraints that have been defined for the step
    ' We do not need a sorted array here - only while loading
    ' the constraints in the list box
    If intConstraintType = gintPreStep Then
        vntConstraints = gcConstraints.PreConstraintsForStep( _
                cStepRecord.StepId, cStepRecord.VersionNo, False)
    Else
        vntConstraints = gcConstraints.PostConstraintsForStep( _
                cStepRecord.StepId, cStepRecord.VersionNo, False)
    End If
    
    If Not IsEmpty(vntConstraints) And IsArray(vntConstraints) Then
        For intIndex = LBound(vntConstraints) To UBound(vntConstraints)
                
            Set cConsRec = vntConstraints(intIndex)
            
            intConsIndex = InList(lstConstraints, SearchId:=cConsRec.ConstraintId)
            
            If intConsIndex >= 0 Then
                If intConsIndex <> cConsRec.SequenceNo Then
                    ' If the sequence of the constraint has
                    ' changed, update it
                    cConsRec.SequenceNo = intConsIndex
                    gcConstraints.Modify cConsRec
                End If
            Else
                ' The constraint does not exist in the list box,
                ' meaning that it has been deleted by the user
                gcConstraints.Delete cConsRec
            End If
        Next intIndex
    End If
    
    ' Call a procedure to insert new constraint records,
    ' if any were created by the user
    Call InsertNewConstraints(lstConstraints, _
            lstGlobals, cStepRecord, intConstraintType)
    Exit Sub
    
WriteConstraintsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errUnableToWriteChanges, mstrModuleName & "WriteConstraints", _
        LoadResString(errUnableToWriteChanges)
    
End Sub
Public Sub CommitGlobalConstraints(chkConstraints As Object, _
        ByVal cStepRecord As cStep)
    ' Saves the constraint records for the passed in global step
    
    Dim intIndex As Integer
    Dim vntConstraints As Variant
    Dim cConsRec As cConstraint
    
    On Error GoTo CommitGlobalConstraintsErr
    
    ' Call a function that will return all the Pre- or post-
    ' execution constraints that have been defined for the step
    vntConstraints = gcConstraints.ConstraintsForStep( _
            cStepRecord.StepId, cStepRecord.VersionNo, _
            intConstraintType:=0, blnSort:=False, _
            blnGlobal:=True, blnGlobalConstraintsOnly:=True)
    
    For intIndex = chkConstraints.LBound To chkConstraints.UBound
        Set cConsRec = GetConstraint(vntConstraints, intIndex)
        If chkConstraints(intIndex).Value = vbUnchecked Then
            If Not cConsRec Is Nothing Then
                ' A constraint that had been defined for the
                ' global step has been deleted
                gcConstraints.Delete cConsRec
            End If
        Else
            If cConsRec Is Nothing Then
                ' A new constraint was created
                ' Insert a constraint record for the global step
                ' with the constraint type equal to the index of
                ' the checkbox
                Call AddGlobalConstraint(cStepRecord, intIndex)
            End If
        End If
    Next intIndex

    Exit Sub
    
CommitGlobalConstraintsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errUnableToWriteChanges, mstrModuleName & "CommitGlobalConstraints", _
        LoadResString(errUnableToWriteChanges)
    
End Sub

Private Function GetConstraint( _
        ByVal vntConstraints As Variant, _
        ByVal intConsType As Integer) As cConstraint
    ' Returns a constraint record of the passed in type,
    ' if it exists in the array
    
    Dim cConsRec As cConstraint
    Dim intIndex As Integer
    
    On Error GoTo GetConstraintErr
    
    ' Initialize the default return value
    Set GetConstraint = Nothing
    
    If Not IsEmpty(vntConstraints) And IsArray(vntConstraints) Then
        For intIndex = LBound(vntConstraints) To _
                UBound(vntConstraints)

            Set cConsRec = vntConstraints(intIndex)
            
            If cConsRec.ConstraintType = intConsType Then
                Set GetConstraint = cConsRec
                Exit Function
            End If

        Next intIndex
    End If

    Exit Function
    
GetConstraintErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errUnableToWriteChanges, mstrModuleName & "GetConstraint", _
        LoadResString(errUnableToWriteChanges)
    
End Function
Private Sub ExecuteConstraintMethod(lstGlobals As ListBox, _
        lstConstraints As ListBox, _
        ByVal intMethod As Integer)

    ' Updates the constraints list box, depending on the
    ' operation selected by the user
    ' Note : the updates are not committed to the database -
    ' the updates will be carried out when the user commits the
    ' step record
    Dim intTemp As Integer
    Dim cArrStepIds As cVectorLng
    
    On Error GoTo ExecuteConstraintMethodErr
    gstrSource = mstrModuleName & "ExecuteConstraintMethod"
    
    ' A global step has to be selected before it can be added
    ' as a constraint. Similarly, an existing constraint has
    ' to be selected before a delete or a move can be done on
    ' it.
    If intMethod = gintAdd And _
            StringEmpty(lstGlobals.Text) Then
        Exit Sub
    ElseIf intMethod <> gintAdd And _
            StringEmpty(lstConstraints.Text) Then
        Exit Sub
    End If
    
    If lstConstraints.Index = gintPreStep Then
        Set cArrStepIds = cPreStepIds
    Else
        Set cArrStepIds = cPostStepIds
    End If
    
    BugAssert Not cArrStepIds Is Nothing, "Constraints uninitialized!"
    
    ' Populate/edit the constraints list box, depending on the
    ' action being performed
    Select Case intMethod
        Case gintAdd
            ' Add the global to the list of constraints only
            ' if has not been already defined as one
            If cArrStepIds.Find(lstGlobals.ItemData( _
                    lstGlobals.ListIndex)) = -1 Then
                lstConstraints.AddItem lstGlobals.Text, _
                        lstConstraints.ListCount
                lstConstraints.ItemData(lstConstraints.NewIndex) = 0
                
                ' Add the step id for the new constraint to the
                ' array
                cArrStepIds(lstConstraints.NewIndex) = _
                        lstGlobals.ItemData(lstGlobals.ListIndex)
            End If
            
        Case gintDelete
            intTemp = lstConstraints.ListIndex
            lstConstraints.RemoveItem lstConstraints.ListIndex
            ' Remove the step id for the deleted constraint
            ' from the array
            cArrStepIds.Delete intTemp
            
            If intTemp - 1 >= 0 Then
                lstConstraints.ListIndex = intTemp - 1
            ElseIf intTemp <= lstConstraints.ListCount - 1 Then
                lstConstraints.ListIndex = intTemp
            End If
    
        Case gintMoveUp
            intTemp = lstConstraints.ListIndex
            If intTemp - 1 >= 0 Then
                ' The move is valid
                Call SwapListEntries(lstConstraints, _
                        intTemp, _
                        intTemp - 1)
                ' Restore the list box selection to the same
                ' item the user was on
                lstConstraints.ListIndex = intTemp - 1
                
                ' Reposition the step ids in the array to
                ' ensure that it remains in sync with the list
                cArrStepIds.MoveUp intTemp
            End If
    
        Case gintMoveDown
            intTemp = lstConstraints.ListIndex
            If intTemp + 1 <= _
                    lstConstraints.ListCount Then
                ' The move is valid
                Call SwapListEntries(lstConstraints, _
                        intTemp, _
                        intTemp + 1)
                ' Restore the list box selection to the same
                ' item the user was on
                lstConstraints.ListIndex = intTemp + 1
                
                ' Reposition the step ids in the array to
                ' ensure that it remains in sync with the list
                cArrStepIds.MoveDown intTemp
            End If
            
        Case Else
            ' This should never happen
            Debug.Assert intMethod < gintMoveDown
    End Select
    
    Exit Sub
    
ExecuteConstraintMethodErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errExecuteConstraintMethodFailed, gstrSource, _
        LoadResString(errExecuteConstraintMethodFailed)
    
End Sub
Public Sub EnableGlobalCommands(lstGlobals As ListBox, _
        lstConstraints As ListBox, _
        tblCommands As Toolbar)
        
    Dim cArrStepIds As cVectorLng
    
    On Error GoTo EnableGlobalCommandsErr
    
    If lstConstraints.Index = gintPreStep Then
        Set cArrStepIds = cPreStepIds
    Else
        Set cArrStepIds = cPostStepIds
    End If
    
    ' The only operations permitted on a global step is to
    ' add it as a constraint
    
    If lstGlobals.ListIndex > -1 Then
        If cArrStepIds.Find(lstGlobals.ItemData( _
                lstGlobals.ListIndex)) = -1 Then
            ' The global can be added to the list of constraints only
            ' if has not been already defined as one
            tblCommands.Buttons(gintAdd).Enabled = True
        Else
            tblCommands.Buttons(gintAdd).Enabled = False
        End If
    End If
            
    tblCommands.Buttons(gintDelete).Enabled = False
    tblCommands.Buttons(gintMoveUp).Enabled = False
    tblCommands.Buttons(gintMoveDown).Enabled = False

    Exit Sub
    
EnableGlobalCommandsErr:

    ' Log the VB error code
    Call LogErrors(Errors)
    ' Resume
    Resume Next
    
End Sub
Private Sub LoadConstraintsInList(lstConstraint As ListBox, _
        vntConstraints As Variant, _
        Optional cArrStepIds As cVectorLng)
    ' Loads all the step constraint records in the passed
    ' in list box
    ' Optionally, it loads the passed in vector with all the
    ' step ids
    
    Dim cConsRec As cConstraint
    Dim cStepRec As cStep
    Dim intIndex As Integer
    Dim strListText As String
    
    On Error GoTo LoadConstraintsInListErr
    
    If Not IsEmpty(vntConstraints) And IsArray(vntConstraints) Then
        For intIndex = LBound(vntConstraints) To _
                UBound(vntConstraints)
                
            Set cConsRec = vntConstraints(intIndex)
            
            ' Query the step record for the selected constraint
            Set cStepRec = New cStep
            Set cStepRec = gcSteps.QueryStep(cConsRec.GlobalStepId)
            
            ' and load it into the list box
            strListText = GetStepNodeText(cStepRec)
            lstConstraint.AddItem strListText, _
                    cConsRec.SequenceNo
            lstConstraint.ItemData(lstConstraint.NewIndex) = _
                    cConsRec.ConstraintId
            
            ' Add the step id to the array
            If Not cArrStepIds Is Nothing Then
                cArrStepIds(lstConstraint.NewIndex) = cStepRec.StepId
            End If
            
        Next intIndex
    End If
        
    Exit Sub
    
LoadConstraintsInListErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "LoadConstraintsInList"
    Err.Raise vbObjectError + errLoadConstraintsInListFailed, gstrSource, _
        LoadResString(errLoadConstraintsInListFailed)
    
End Sub
Public Sub DeleteStepConstraints(ByVal cDeleteStep As cStep, _
        Optional ByVal ArchivedFlag As Boolean = False)
    ' Deletes/Unloads all the step constraint records for the passed
    ' in step
    
    Dim vntConstraints As Variant
    Dim cConsRec As cConstraint
    Dim intIndex As Integer
    
    On Error GoTo DeleteStepConstraintsErr
    
    ' Call a function that will return all the constraints
    ' that have been defined for the step - set the constraint
    ' type parameter to False to indicate that we want both
    ' pre and post-execution constraints for the step
    ' The value of the blnGlobal parameter will depend on the
    ' type of the passed in step
    vntConstraints = gcConstraints.ConstraintsForStep( _
            cDeleteStep.StepId, cDeleteStep.VersionNo, _
            intConstraintType:=False, blnSort:=False, _
            blnGlobal:=IsGlobal(StepClass:=cDeleteStep))
    
    If Not IsEmpty(vntConstraints) And IsArray(vntConstraints) Then
        For intIndex = LBound(vntConstraints) To _
                UBound(vntConstraints)
                
            Set cConsRec = vntConstraints(intIndex)
            
            If ArchivedFlag Then
                ' Just unload the constraint from the array
                gcConstraints.UnloadConstraint cConsRec
            Else
                ' Delete the constraint from the array of constraints
                gcConstraints.Delete cConsRec
            End If
            
            ' If the step is a global step, then the remaining
            ' constraints might have to be reordered
            If IsGlobal(StepClass:=cDeleteStep) Then
                Call BumpUpConstraints(cConsRec)
            End If
            
        Next intIndex
    End If
        
    Exit Sub
    
DeleteStepConstraintsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "DeleteStepConstraints"
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteStepConstraintsFailed, _
            gstrSource, _
            LoadResString(errDeleteStepConstraintsFailed)
    
End Sub
Private Sub BumpUpConstraints(ByVal cDeleteCons As cConstraint)
    ' Decrements the sequence numbers of all the constraints
    ' that appear after the passed in constraint
    
    Dim vntConstraints As Variant
    Dim cConsRec As cConstraint
    Dim intIndex As Integer
    Dim intSequence As Integer
    
    On Error GoTo BumpUpConstraintsErr
    
    ' Call a function that will return all the constraints
    ' that have been defined for the step - set the constraint
    ' type parameter to the type on the constraint record
    ' This will ensure that only the constraints that are
    ' currently being processed will be retrieved
    vntConstraints = gcConstraints.ConstraintsForStep( _
            cDeleteCons.StepId, cDeleteCons.VersionNo, _
            intConstraintType:=cDeleteCons.ConstraintType, _
            blnSort:=True, blnGlobal:=False)
    
    intSequence = cDeleteCons.SequenceNo
    
    If Not IsEmpty(vntConstraints) And IsArray(vntConstraints) Then
        For intIndex = LBound(vntConstraints) To _
                UBound(vntConstraints)
                
            Set cConsRec = vntConstraints(intIndex)
            
            If cConsRec.SequenceNo > intSequence Then
                ' Decrement the sequence number for the constraint
                ' by 1
                cConsRec.SequenceNo = cConsRec.SequenceNo - 1
                Call gcConstraints.Modify(cConsRec)
            End If
        Next intIndex
    End If
        
    Exit Sub
    
BumpUpConstraintsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "BumpUpConstraints"
    On Error GoTo 0
    Err.Raise vbObjectError + errUpdateSequenceFailed, _
            gstrSource, _
            LoadResString(errUpdateSequenceFailed)
    
End Sub
Public Sub LoadGlobalSteps(lstGlobals As ListBox, _
        lstConstraints As ListBox, _
        lngWorkspaceId As Long)

    Dim cStepRec As cStep
    Dim strGlobal As String
    Dim nodNextSibling As Node
    Dim intConstraintType As Integer
    
    On Error GoTo LoadGlobalStepsErr
    gstrSource = mstrModuleName & "LoadGlobalSteps"
    
    strGlobal = LabelGlobalStep(lngWorkspaceId)
    intConstraintType = lstConstraints.Index
    
    If frmMain.treStepView.Nodes(strGlobal).Children > 0 Then
        Set nodNextSibling = frmMain.treStepView.Nodes(strGlobal).Child
        
        Do
            If nodNextSibling Is Nothing Then
                Exit Do
            End If
        
            Set cStepRec = New cStep
            cStepRec.StepId = MakeIdentifierValid( _
                    nodNextSibling.Key)
            Set cStepRec = gcSteps.QueryStep(cStepRec.StepId)
            If IsGlobal(StepClass:=cStepRec) Then
'                If cStepRec.GlobalRunMethod = gintRunGlobalPreOrPost Then
                lstGlobals.AddItem nodNextSibling.Text
                lstGlobals.ItemData(lstGlobals.NewIndex) = _
                        cStepRec.StepId
'                End If
                ' If the global constraint type matches the
                ' constraint being displayed, add it to the
                ' list box
'                If cStepRec.GlobalRunMethod = intConstraintType Then
'                    lstConstraints.AddItem nodNextSibling.Text
'                    lstGlobals.ItemData(lstGlobals.NewIndex) = _
'                            cStepRec.StepId
'                End If
            End If
            Set cStepRec = Nothing
            Set nodNextSibling = nodNextSibling.Next
        
        Loop While True
    End If
    
    Exit Sub
    
LoadGlobalStepsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadGlobalStepsFailed, gstrSource, _
        LoadResString(errLoadGlobalStepsFailed)
    
End Sub

Public Sub LoadPostExecutionSteps(lstPost As ListBox, _
        ByVal lngStepId As Long, _
        ByVal strVersion As String)
    ' Will populate the passed in list box with all the Post-
    ' execution constraints for the step
        
    Dim vntPostExecutionSteps As Variant
        
    On Error GoTo LoadPostExecutionStepsErr
    
    ' Call a function that will return all the post-execution
    ' constraints that have been defined for the step
    vntPostExecutionSteps = gcConstraints.PostConstraintsForStep( _
            lngStepId, strVersion, True)
    
    ' Call a procedure that will load all the passed in
    ' constraints into the list
    Call LoadConstraintsInList(lstPost, vntPostExecutionSteps, _
                cPostStepIds)
    Exit Sub

LoadPostExecutionStepsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadPostExecutionStepsFailed, mstrModuleName & "LoadPostExecutionSteps", _
        LoadResString(errLoadPostExecutionStepsFailed)
    
End Sub
Public Sub LoadWspPostExecSteps(lstWspPost As ListBox, _
        ByVal lngWorkspaceId As Long)
    ' Will populate the passed in list box with all the Post-
    ' execution constraints that have been defined for the
    ' workspace
        
    Dim vntWspPostExecSteps As Variant
        
    On Error GoTo LoadWspPostExecStepsErr
    
    ' Call a function that will return all the post-execution
    ' constraints that have been defined for the step
    vntWspPostExecSteps = gcConstraints.PostConstraintsForWsp( _
            lngWorkspaceId, True)
    
    ' Call a procedure that will load all the passed in
    ' constraints into the list
    Call LoadConstraintsInList(lstWspPost, vntWspPostExecSteps)
    Exit Sub

LoadWspPostExecStepsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadWspPostExecStepsFailed, mstrModuleName & "LoadWspPostExecSteps", _
        LoadResString(errLoadWspPostExecStepsFailed)
    
End Sub
Public Sub LoadWspPreExecSteps(lstWspPre As ListBox, _
        ByVal lngWorkspaceId As Long)
    ' Will populate the passed in list box with all the Pre-
    ' execution constraints that have been defined for the
    ' workspace
        
    Dim vntWspPreExecSteps As Variant
        
    On Error GoTo LoadWspPreExecStepsErr
    
    ' Call a function that will return all the Pre-execution
    ' constraints that have been defined for the step
    vntWspPreExecSteps = gcConstraints.PreConstraintsForWsp( _
            lngWorkspaceId, True)
    
    ' Call a procedure that will load all the passed in
    ' constraints into the list
    Call LoadConstraintsInList(lstWspPre, vntWspPreExecSteps)
    Exit Sub

LoadWspPreExecStepsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadWspPreExecStepsFailed, mstrModuleName & "LoadWspPreExecSteps", _
        LoadResString(errLoadWspPreExecStepsFailed)
    
End Sub

Public Sub LoadPreExecutionSteps(lstPre As ListBox, _
        ByVal lngStepId As Long, _
        ByVal strVersion As String)
    ' Will populate the passed in list box with all the Pre-
    ' execution constraints for the step
        
    Dim vntPreExecutionSteps As Variant
        
    On Error GoTo LoadPreExecutionStepsErr
    
    ' Call a function that will return all the Pre-execution
    ' constraints that have been defined for the step
    vntPreExecutionSteps = gcConstraints.PreConstraintsForStep( _
            lngStepId, strVersion, True)
    
    ' Call a procedure that will load all the passed in
    ' constraints into the list
    Call LoadConstraintsInList(lstPre, vntPreExecutionSteps, _
            cPreStepIds)

    Exit Sub

LoadPreExecutionStepsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "LoadPreExecutionSteps"
    Err.Raise vbObjectError + errLoadPreExecutionStepsFailed, gstrSource, _
        LoadResString(errLoadPreExecutionStepsFailed)
    
End Sub
Public Sub LoadConstraintsForGlobal(chkConstraints As Object, _
        ByVal lngStepId As Long, _
        ByVal strVersion As String)
    ' Will initialize the checkboxes based on the constraints
    ' that have been defined for the step
        
    Dim intIndex As Integer
    Dim vntConstraints As Variant
    Dim cConsRec As cConstraint
    
    On Error GoTo LoadConstraintsForGlobalErr
    gstrSource = mstrModuleName & "LoadConstraintsForGlobal"
    
    ' Call a function that will return all the Pre- or post-
    ' execution constraints that have been defined for the step
    vntConstraints = gcConstraints.ConstraintsForStep( _
            lngStepId, strVersion, _
            intConstraintType:=0, blnSort:=False, _
            blnGlobal:=True, blnGlobalConstraintsOnly:=True)
    
    If Not IsEmpty(vntConstraints) And IsArray(vntConstraints) Then
        For intIndex = LBound(vntConstraints) To _
                UBound(vntConstraints)

            Set cConsRec = vntConstraints(intIndex)
            
            If cConsRec.ConstraintType >= chkConstraints.LBound And _
                    cConsRec.ConstraintType <= chkConstraints.UBound Then
                ' If a constraint record exists, set the property
                ' for the corresponding checkbox to Checked
                chkConstraints(cConsRec.ConstraintType).Value = vbChecked
            Else
                ' Write an error to the log and proceed
                WriteError errConstraintTypeInvalid, gstrSource
                On Error GoTo 0
                Err.Raise vbObjectError + errLoadDataFailed, gstrSource, _
                        LoadResString(errLoadDataFailed)
            End If
            
        Next intIndex
    End If

    Exit Sub

LoadConstraintsForGlobalErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "LoadConstraintsForGlobal"
    Err.Raise vbObjectError + errLoadDataFailed, gstrSource, _
            LoadResString(errLoadDataFailed)
    
End Sub
