Attribute VB_Name = "RunCommon"
'  FILE:       RunCommon.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains common functions that are used during the execution
'              of a workspace.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this class
Private Const mstrModuleName As String = ".RunCommon."


Public Function GetInstanceItValue(cInstanceRec As cInstance) As String

    ' Returns the iterator value for the instance, if an
    ' iterator has been defined for it
    Dim cStepIt As cRunColIt
    Dim cRunIterator As cRunItNode
    
    On Error GoTo GetInstanceItValueErr

    ' Since we create a dummy instance for Disabled and Pending steps,
    ' doesn't make sense to look at their iterators
    If cInstanceRec.Status <> gintDisabled And cInstanceRec.Status <> gintPending Then
        Set cStepIt = cInstanceRec.Iterators
        
        If Not StringEmpty(cInstanceRec.Step.IteratorName) Then
            If cStepIt.Count > 0 Then
                Set cRunIterator = cStepIt(0)
                BugAssert cRunIterator.IteratorName = cInstanceRec.Step.IteratorName, _
                        "The first iterator in the collection is the " & _
                        "one that has been defined for the step."
                If cRunIterator.IteratorName = cInstanceRec.Step.IteratorName Then
                    GetInstanceItValue = cRunIterator.Value
                Else
                    GetInstanceItValue = gstrEmptyString
                End If
            Else
                GetInstanceItValue = gstrEmptyString
            End If
        Else
            GetInstanceItValue = gstrEmptyString
        End If
    Else
        GetInstanceItValue = gstrEmptyString
    End If
    
    Exit Function
    
GetInstanceItValueErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetInstanceItValue"
    Err.Raise vbObjectError + errProgramError, gstrSource, _
            LoadResString(errProgramError)
    
End Function


